package com.galaxis.wms.filter;

import com.galaxis.wms.GlobalConstant;
import com.galaxis.wms.utils.ExchangeUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.cloud.gateway.support.ServerWebExchangeUtils;
import org.springframework.core.Ordered;
import org.springframework.http.codec.HttpMessageReader;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.server.HandlerStrategies;
import org.springframework.web.reactive.function.server.ServerRequest;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;

/**
 * 作者：lizw <br/>
 * 创建时间：2022/03/17 10:05 <br/>
 */
@Component
@Slf4j
public class CacheRequestBodyGlobalFilter implements GlobalFilter, Ordered {
    private static final int ORDER = Ordered.HIGHEST_PRECEDENCE;
    private static final List<HttpMessageReader<?>> MESSAGE_READERS = HandlerStrategies.withDefaults().messageReaders();

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        return ServerWebExchangeUtils.cacheRequestBody(exchange, serverHttpRequest -> {
            final ServerWebExchange newExchange = ExchangeUtils.modifyRequestHeaders(exchange, serverHttpRequest);
            // 获取 Request Body 参考 ReadBodyPredicateFactory
            final Mono<String> mono = ServerRequest.create(newExchange, MESSAGE_READERS)
                    .bodyToMono(String.class)
                    .doOnNext(body -> {
                        try {
                            // log.info("body -> {}", body);
                            newExchange.getAttributes().put(GlobalConstant.CACHED_REQUEST_BODY_STR_ATTR, body);
                        } catch (Exception e) {
                            log.error("缓存请求body处理失败", e);
                        }
                    });
            // 合并Mono
            return Flux.merge(mono, chain.filter(newExchange)).then();
        });
    }

    @Override
    public int getOrder() {
        return ORDER;
    }
}
